from bpm_ai_core.llm.common.llm import LLM
from bpm_ai_core.llm.common.message import ToolCallsMessage
from bpm_ai_core.llm.common.tool import Tool
from bpm_ai_core.prompt.prompt import Prompt
from bpm_ai_core.speech.stt.stt import STTModel
from bpm_ai_core.tracing.decorators import trace

from bpm_ai.common.multimodal import prepare_audio
from bpm_ai.translate.schema import get_translation_output_schema


@trace("bpm-ai-translate")
def run_translate(
    llm: LLM,
    input_data: dict[str, str | dict],
    target_language: str,
    stt: STTModel | None = None
) -> dict:
    tool = Tool.from_callable(
        "store_translation",
        f"Stores the finished translation into {target_language}.",
        args_schema=get_translation_output_schema(input_data, target_language),
        callable=lambda **x: x
    )

    #input_data = prepare_images(input_data)  todo enable once GPT-4V is stable
    input_data = prepare_audio(input_data, stt)

    prompt = Prompt.from_file(
        "translate",
        input=input_data
    )

    result = llm.predict(prompt, tools=[tool])

    if isinstance(result, ToolCallsMessage):
        return result.tool_calls[0].invoke()
    else:
        return {}






from bpm_ai_core.speech.stt.stt import STTModel
from bpm_ai_core.util.audio import is_supported_audio_file
from bpm_ai_core.util.image import is_supported_img_file


def prepare_images(input_data: dict):
    return {
        k: f"[# image {v} #]"
        if (isinstance(v, str) and is_supported_img_file(v))
        else v for k, v in input_data.items()
    }


def prepare_audio(input_data: dict, stt: STTModel | None):
    return {
        k: stt.transcribe(v)
        if (stt and isinstance(v, str) and is_supported_audio_file(v))
        else v for k, v in input_data.items()
    }


from typing import Any

from bpm_ai_core.llm.common.llm import LLM
from bpm_ai_core.llm.common.message import ToolCallsMessage
from bpm_ai_core.llm.common.tool import Tool
from bpm_ai_core.prompt.prompt import Prompt
from bpm_ai_core.speech.stt.stt import STTModel
from bpm_ai_core.tracing.decorators import trace

from bpm_ai.common.json_utils import json_to_md
from bpm_ai.common.multimodal import prepare_audio
from bpm_ai.decide.schema import get_cot_decision_output_schema, get_decision_output_schema


@trace("bpm-ai-decide")
def run_decide(
    llm: LLM,
    input_data: dict[str, str | dict],
    instructions: str,
    output_type: str,
    possible_values: list[Any] | None = None,
    strategy: str | None = None,
    stt: STTModel | None = None
) -> dict:
    if strategy == 'cot':
        output_schema = get_cot_decision_output_schema(output_type, possible_values)
    else:
        output_schema = get_decision_output_schema(output_type, possible_values)

    tool = Tool.from_callable(
        "store_decision",
        "Stores the final decision value and corresponding reasoning.",
        args_schema=output_schema,
        callable=lambda **x: x
    )

    #input_data = prepare_images(input_data)  todo enable once GPT-4V is stable
    input_data = prepare_audio(input_data, stt)

    input_md = json_to_md(input_data)

    prompt = Prompt.from_file(
        "decide",
        context=input_md,
        task=instructions,
        output_type=output_type,
        possible_values=possible_values,
        strategy=strategy
    )

    result = llm.predict(prompt, tools=[tool])

    if isinstance(result, ToolCallsMessage):
        return result.tool_calls[0].invoke()
    else:
        return {}






from bpm_ai_core.llm.common.llm import LLM
from bpm_ai_core.llm.common.message import ToolCallsMessage
from bpm_ai_core.llm.common.tool import Tool
from bpm_ai_core.prompt.prompt import Prompt
from bpm_ai_core.speech.stt.stt import STTModel
from bpm_ai_core.tracing.decorators import trace

from bpm_ai.common.json_utils import json_to_md
from bpm_ai.common.multimodal import prepare_audio


@trace("bpm-ai-extract")
def run_extract(
    llm: LLM,
    input_data: dict[str, str | dict],
    output_schema: dict[str, str | dict],
    repeated: bool = False,
    repeated_description: str = "",
    stt: STTModel | None = None
) -> dict:
    def transform_result(**extracted):
        def empty_to_none(v):
            return None if v in ["", "null"] else v

        if repeated and "entities" in extracted.keys():
            extracted = extracted["entities"]

        if isinstance(extracted, list):
            return [transform_result(**d) for d in extracted]
        else:
            return {k: empty_to_none(v) for k, v in extracted.items()}

    tool = Tool.from_callable(
        "information_extraction",
        f"Extracts the relevant {'entities' if repeated else 'information'} from the passage.",
        args_schema={
            "entities": {"type": "array", "description": repeated_description, "items": output_schema}
        } if repeated else output_schema,
        callable=transform_result
    )

    #input_data = prepare_images(input_data)  todo enable once GPT-4V is stable
    input_data = prepare_audio(input_data, stt)

    input_md = json_to_md(input_data)

    prompt = Prompt.from_file("extract", input=input_md)

    result = llm.predict(prompt, tools=[tool])

    if isinstance(result, ToolCallsMessage):
        return result.tool_calls[0].invoke()
    else:
        return {}






from bpm_ai_core.llm.common.llm import LLM
from bpm_ai_core.llm.common.message import ToolCallsMessage
from bpm_ai_core.llm.common.tool import Tool
from bpm_ai_core.prompt.prompt import Prompt
from bpm_ai_core.speech.stt.stt import STTModel
from bpm_ai_core.tracing.decorators import trace

from bpm_ai.common.json_utils import json_to_md
from bpm_ai.common.multimodal import prepare_audio


@trace("bpm-ai-generic")
def run_generic(
    llm: LLM,
    input_data: dict[str, str | dict],
    instructions: str,
    output_schema: dict[str, str | dict],
    stt: STTModel | None = None
) -> dict:
    tool = Tool.from_callable(
        "store_task_result",
        "Stores the result of the task.",
        args_schema=output_schema,
        callable=lambda **x: x
    )

    #input_data = prepare_images(input_data)  todo enable once GPT-4V is stable
    input_data = prepare_audio(input_data, stt)

    input_md = json_to_md(input_data)

    prompt = Prompt.from_file(
        "generic",
        context=input_md,
        task=instructions,
    )

    result = llm.predict(prompt, tools=[tool])

    if isinstance(result, ToolCallsMessage):
        return result.tool_calls[0].invoke()
    else:
        return {}



